@@ -10,7 +10,7 @@ from typing import Any, Callable, Dict, List, Tuple
 import hal
 import wpilib
 
-from networktables import NetworkTables, NetworkTableEntry
+from ntcore import NetworkTableInstance, NetworkTableEntry
 
 # from wpilib.shuffleboard import Shuffleboard
 
@@ -78,18 +78,24 @@ class MagicRobot(wpilib.RobotBase):
 
         self.__done = False
 
-    def _simulationInit(self):
+        # cache these
+        self.__is_ds_attached = wpilib.DriverStation.isDSAttached
+        self.__sd_update = wpilib.SmartDashboard.updateValues
+        self.__lv_update = wpilib.LiveWindow.updateValues
+        # self.__sf_update = Shuffleboard.update
+
+    def _simulationInit(self) -> None:
         pass
 
-    def _simulationPeriodic(self):
+    def _simulationPeriodic(self) -> None:
         pass
 
-    def __simulationPeriodic(self):
+    def __simulationPeriodic(self) -> None:
         hal.simPeriodicBefore()
         self._simulationPeriodic()
         hal.simPeriodicAfter()
 
-    def robotInit(self):
+    def robotInit(self) -> None:
         """
         .. warning:: Internal API, don't override; use :meth:`createObjects` instead
         """
@@ -109,7 +115,7 @@ class MagicRobot(wpilib.RobotBase):
         self.__lv_update = wpilib.LiveWindow.updateValues
         # self.__sf_update = Shuffleboard.update
 
-        self.__nt = NetworkTables.getTable("/robot")
+        self.__nt = NetworkTableInstance.getDefault().getTable("/robot")
 
         self.__nt_put_is_ds_attached = self.__nt.getEntry("is_ds_attached").setBoolean
         self.__nt_put_mode = self.__nt.getEntry("mode").setString
@@ -119,7 +125,9 @@ class MagicRobot(wpilib.RobotBase):
 
         self.watchdog = SimpleWatchdog(self.control_loop_wait_time)
 
-        self.__periodics = [(self.robotPeriodic, "robotPeriodic()")]
+        self.__periodics: List[Tuple[Callable[[], None], str]] = [
+            (self.robotPeriodic, "robotPeriodic()"),
+        ]
 
         if self.isSimulation():
             self._simulationInit()
@@ -369,6 +377,7 @@ class MagicRobot(wpilib.RobotBase):
 
     def endCompetition(self) -> None:
         self.__done = True
+        self._automodes.endCompetition()
 
     def autonomous(self) -> None:
         """
@@ -426,9 +435,17 @@ class MagicRobot(wpilib.RobotBase):
             self.onException(forceReport=True)
         watchdog.addEpoch("disabledInit()")
 
+        refreshData = wpilib.DriverStation.refreshData
+        DSControlWord = wpilib.DSControlWord
+
         with NotifierDelay(self.control_loop_wait_time) as delay:
-            while not self.__done and self.isDisabled():
-                if ds_attached != self.__is_ds_attached():
+            while not self.__done:
+                refreshData()
+                cw = DSControlWord()
+                if cw.isEnabled():
+                    break
+
+                if ds_attached != cw.isDSAttached():
                     ds_attached = not ds_attached
                     self.__nt_put_is_ds_attached(ds_attached)
 
@@ -476,9 +493,15 @@ class MagicRobot(wpilib.RobotBase):
         watchdog.addEpoch("teleopInit()")
 
         observe = hal.observeUserProgramTeleop
+        refreshData = wpilib.DriverStation.refreshData
+        isTeleopEnabled = wpilib.DriverStation.isTeleopEnabled
 
         with NotifierDelay(self.control_loop_wait_time) as delay:
-            while not self.__done and self.isOperatorControlEnabled():
+            while not self.__done:
+                refreshData()
+                if not isTeleopEnabled():
+                    break
+
                 observe()
                 try:
                     self.teleopPeriodic()
@@ -518,8 +541,16 @@ class MagicRobot(wpilib.RobotBase):
             self.onException(forceReport=True)
         watchdog.addEpoch("testInit()")
 
+        refreshData = wpilib.DriverStation.refreshData
+        DSControlWord = wpilib.DSControlWord
+
         with NotifierDelay(self.control_loop_wait_time) as delay:
-            while not self.__done and self.isTest() and self.isEnabled():
+            while not self.__done:
+                refreshData()
+                cw = DSControlWord()
+                if not (cw.isTest() and cw.isEnabled()):
+                    break
+
                 hal.observeUserProgramTest()
                 try:
                     self.testPeriodic()
@@ -538,10 +569,10 @@ class MagicRobot(wpilib.RobotBase):
                 delay.wait()
                 watchdog.reset()
 
-        lw.setEnabled(False)
+        wpilib.LiveWindow.setEnabled(False)
         # Shuffleboard.disableActuatorWidgets()
 
-    def _on_mode_enable_components(self):
+    def _on_mode_enable_components(self) -> None:
         # initialize things
         for _, component in self._components:
             on_enable = getattr(component, "on_enable", None)
@@ -551,7 +582,7 @@ class MagicRobot(wpilib.RobotBase):
                 except:
                     self.onException(forceReport=True)
 
-    def _on_mode_disable_components(self):
+    def _on_mode_disable_components(self) -> None:
         # deinitialize things
         for _, component in self._components:
             on_disable = getattr(component, "on_disable", None)
@@ -561,7 +592,7 @@ class MagicRobot(wpilib.RobotBase):
                 except:
                     self.onException(forceReport=True)
 
-    def _create_components(self):
+    def _create_components(self) -> None:
 
         #
         # TODO: Will need to inject into any autonomous mode component
@@ -580,7 +611,7 @@ class MagicRobot(wpilib.RobotBase):
 
         # - Iterate over class variables with type annotations
         # .. this hack is necessary for pybind11 based modules
-        sys.modules["pybind11_builtins"] = types.SimpleNamespace()
+        sys.modules["pybind11_builtins"] = types.SimpleNamespace()  # type: ignore
 
         for m, ctyp in typing.get_type_hints(cls).items():
             # Ignore private variables
